##
## R code for performing parameter optimization on epitope-to-MHC binding
## data set as presented in the manuscript
##
##    J. Palme, S. Hochreiter, and U. Bodenhofer.
##    KeBABS: an R package for kernel-based analysis of biological sequences
##    Bioinformatics 31:2574-2576, 2015. DOI: 10.1093/bioinformatics/btv176.
##
##  NOTE: The code in this source file requires at least version 1.1.8 of
##        the 'kebabs' package.
##

## load packages
library(SparseM)
library(kebabs)
packageDescription("kebabs")$Version ## NOTE: at least version 1.1.8 required!

## load R workspace file with data set
load("MHC_I_A0201_Strong.RData")

ls()
a0201strongseqs
table(a0201strongy)

## set random seed for reproducability (omit if not necessary)
# set.seed(123)

## select 40% of samples for parameter optimization
msel <- sample(1:length(a0201strongseqs), 0.4 * length(a0201strongseqs))
mdat <- c(1:length(a0201strongseqs))[-msel]

## define position specific spectrum kernel
specK1ps <- spectrumKernel(k=1, distWeight=1)

## perform parameter optimization (cost parameter C) by 5 times 10-fold
## cross validation
modelParSel <- kbsvm(x=a0201strongseqs[msel], y=a0201strongy[msel],
                     kernel=specK1ps,
                     pkg="kernlab", svm="C-svc",
                     cost=c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1,
                            1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2,
                            2.5, 3, 3.5, 4, 4.5, 5),
                     cross=10, noCross=5,
                     perfParameters="ALL", showProgress=TRUE)

modelSelResult(modelParSel)
performance(modelSelResult(modelParSel))

## perform 10 times 10-fold cross validation with optimal cost parameter
## on remaining 60% of the data set
selCost <- selGridCol(modelSelResult(modelParSel))$cost
selCost

model <- kbsvm(x=a0201strongseqs[mdat], y=a0201strongy[mdat],
               kernel=specK1ps,
               pkg="kernlab", svm="C-svc", cost=selCost,
               cross=10, noCross=10, perfParameters="ALL")

cvResult(model)
